@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.30 2003/11/21 21:03:31 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.31 2003/11/22 19:17:16 fuyu Exp $
  */
 
 #include "config.h"
@@ -36,6 +36,7 @@
 #include "bbs_thread_ui.h"
 #include "bbs_thread_view.h"
 #include "boardlist_ui.h"
+#include "bulletin_board_ui.h"
 #include "icon_label.h"
 
 #include "htmlutils.h"
@@ -136,6 +137,8 @@ typedef struct _RendererContext
   int number_of_gt_characters;
   gboolean last_text_is_link;
 
+  gboolean show_mailto_literally;
+
   GHashTable *tag_table;
 
   GtkTextTag *thread_title_tag;
@@ -201,6 +204,7 @@ PopupContext global_popup_context =
 
     0,					/* number_of_gt_characters */
     FALSE,				/* last_text_is_link */
+    FALSE,				/* show_mailto_literally */
 
     NULL,				/* tag_table */
 
@@ -802,6 +806,28 @@ link_mouse_release_cb(BBSThreadView *view, GdkEventButton *event,
 #define DEFAULT_LINK_COLOR	"blue"
 
 
+static gboolean
+show_mailto(OchushaApplication *application, OchushaBBSThread *thread)
+{
+  BBSThreadGUIInfo *thread_info = ensure_bbs_thread_info(thread);
+  BulletinBoardGUIInfo *board_info;
+  if (thread_info->show_mailto_mode != THREAD_VIEW_MAILTO_MODE_DEFAULT)
+    return (thread_info->show_mailto_mode
+	    == THREAD_VIEW_MAILTO_MODE_SHOW);
+
+  board_info = ensure_bulletin_board_info(thread->board, application);
+  if (board_info->show_mailto_mode != THREAD_VIEW_MAILTO_MODE_DEFAULT)
+    return (board_info->show_mailto_mode
+	    == THREAD_VIEW_MAILTO_MODE_SHOW);
+
+  if (application->show_mailto_mode != THREAD_VIEW_MAILTO_MODE_DEFAULT)
+    return (application->show_mailto_mode
+	    == THREAD_VIEW_MAILTO_MODE_SHOW);
+
+  return FALSE;
+}
+
+
 static void
 render_bbs_thread(WorkerThread *employee, gpointer args)
 {
@@ -838,6 +864,7 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
 
       0,					/* number_of_gt_characters */
       FALSE,					/* last_text_is_link */
+      show_mailto(application, thread),		/* show_mailto_literally */
 
       tag_table,				/* tag_table */
 
@@ -1051,32 +1078,13 @@ render_response(OchushaBBSThread *thread, gint number,
   bbs_thread_view_append_text(view, buffer, -1);
   bbs_thread_view_append_text(view, LARGE_COLON, 3);
 
-  bbs_thread_view_push_tag(view, context->response_name_tag);
   if (response->mailto == NULL)
     {
+      bbs_thread_view_push_tag(view, context->response_name_tag);
       bbs_thread_view_push_tag(view, context->bold_tag);
       parse_text(&context->handler, context, response->name, -1);
       bbs_thread_view_pop_tags(view, context->bold_tag);
-    }
-  else if (strcmp(response->mailto, "sage") != 0)
-    {
-      gchar *name = simple_string_canon(response->name, -1,
-					context->handler.converter,
-					context->handler.helper);
-      gchar *mailto = convert_string(context->handler.converter,
-				     context->handler.helper,
-				     response->mailto, -1);
-#if 0
-      if (mailto == NULL)
-	{
-	  fprintf(stderr, "XXX: name=%s\n", name);
-	  fprintf(stderr, "XXX: response->name=%s\n", response->name);
-	  fprintf(stderr, "XXX: response->mailto=%s\n", response->mailto);
-	}
-#endif
-      bbs_thread_view_append_text_as_link(view, name, -1, mailto);
-      G_FREE(mailto);
-      G_FREE(name);
+      bbs_thread_view_pop_tags(view, context->response_name_tag);
     }
   else
     {
@@ -1086,11 +1094,30 @@ render_response(OchushaBBSThread *thread, gint number,
       gchar *mailto = convert_string(context->handler.converter,
 				     context->handler.helper,
 				     response->mailto, -1);
-      bbs_thread_view_append_text_as_sage(view, name, -1, mailto);
-      G_FREE(mailto);
-      G_FREE(name);
+      bbs_thread_view_push_tag(view, context->response_name_tag);
+      if (strcmp(response->mailto, "sage") != 0)
+	bbs_thread_view_append_text_as_link(view, name, -1,
+					    (mailto != NULL
+					     ? mailto : _("broken here")));
+      else
+	bbs_thread_view_append_text_as_sage(view, name, -1,
+					    (mailto != NULL
+					     ? mailto : _("broken here")));
+      bbs_thread_view_pop_tags(view, context->response_name_tag);
+      if (context->show_mailto_literally)
+	{
+	  bbs_thread_view_append_text(view, " [", 2);
+	  bbs_thread_view_append_text(view,
+				      (mailto != NULL
+				       ? mailto : _("broken here")), -1);
+	  bbs_thread_view_append_text(view, "]", 1);
+	}
+	    
+      if (name != NULL)
+	G_FREE(name);
+      if (mailto != NULL)
+	G_FREE(mailto);
     }
-  bbs_thread_view_pop_tags(view, context->response_name_tag);
 
   bbs_thread_view_append_text(view, LARGE_COLON, 3);
   parse_text(&context->handler, context, response->date_id, -1);
@@ -1809,6 +1836,9 @@ render_popup_response(PopupContext *popup_context,
   fprintf(stderr, "render_popup_response(): view=%p\n", view);
 #endif
 
+  popup_context->renderer_context.show_mailto_literally
+    = show_mailto(application, thread);
+
   g_signal_connect(G_OBJECT(view), "link_mouse_over",
 		   G_CALLBACK(link_mouse_over_cb), application);
   g_signal_connect(G_OBJECT(view), "link_mouse_out",
