@@ -23,12 +23,13 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.h,v 1.8 2003/11/04 18:09:57 fuyu Exp $
+ * $Id: bbs_thread_ui.h,v 1.9 2003/11/22 19:17:16 fuyu Exp $
  */
 
 #ifndef _BBSTHREAD_UI_H_
 #define _BBSTHREAD_UI_H_
 
+#include "ochusha_ui.h"
 #include "ochusha_bbs_thread.h"
 #include "ochusha_async_buffer.h"
 #include "threadlist_view.h"
@@ -59,6 +60,9 @@ typedef struct _BBSThreadGUIInfo
   /* ɽ(ThreadlistEntryFlagsORȤä) */
   int view_flags;
 
+  /* ᥤ򥹥ɽϤʸȤɽ뤫ɤ */
+  ThreadViewMailtoMode show_mailto_mode;
+
   /* ǽο쥹Ƭդޡ */
   GtkTextMark *last_read;
 
