@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.22 2003/11/21 21:03:31 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.23 2003/11/22 19:17:16 fuyu Exp $
  */
 
 #include "config.h"
@@ -1068,6 +1068,13 @@ boardlist_read_board_element_cb(OchushaBBSTable *table,
 				OchushaApplication *application)
 {
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
+  info->show_mailto_mode
+    = (ThreadViewMailtoMode)ochusha_utils_get_attribute_int(board_attributes,
+							   "show_mailto_mode");
+  if (info->show_mailto_mode < THREAD_VIEW_MAILTO_MODE_DEFAULT
+      || info->show_mailto_mode > THREAD_VIEW_MAILTO_MODE_HIDE)
+    info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_DEFAULT;
+
   info->enable_filter = ochusha_utils_get_attribute_boolean(board_attributes,
 							    "enable_filter");
   info->filter.rule = ochusha_utils_get_attribute_string(board_attributes,
@@ -1131,6 +1138,7 @@ boardlist_write_board_element_cb(OchushaBBSTable *table,
 				 OchushaApplication *application)
 {
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
+  OUTPUT_BOARD_ATTRIBUTE_INT(boardlist_xml, info, show_mailto_mode);
   OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(boardlist_xml, info, enable_filter);
   OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, info, last_name);
   OUTPUT_BOARD_ATTRIBUTE_STRING(boardlist_xml, info, last_mail);
