@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.45 2003/11/19 21:15:57 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.46 2003/11/22 19:17:16 fuyu Exp $
  */
 
 #include "config.h"
@@ -219,6 +219,13 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
 
   info->view_flags = ochusha_utils_get_attribute_int(hash_table, "view_flags");
   info->view_rank = ochusha_utils_get_attribute_int(hash_table, "view_rank");
+  info->show_mailto_mode
+    = (ThreadViewMailtoMode)ochusha_utils_get_attribute_int(hash_table,
+							   "show_mailto_mode");
+  if (info->show_mailto_mode < THREAD_VIEW_MAILTO_MODE_DEFAULT
+      && info->show_mailto_mode > THREAD_VIEW_MAILTO_MODE_HIDE)
+    info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_DEFAULT;
+
   info->view_ignored = ochusha_utils_get_attribute_int(hash_table,
 						       "view_ignored");
   info->last_name = ochusha_utils_get_attribute_string(hash_table,
@@ -260,6 +267,7 @@ threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info,
 				  view_flags);
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, view_rank);
+      OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, show_mailto_mode);
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, view_ignored);
       OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info, last_name);
       OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info, last_mail);
