@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.h,v 1.12 2003/11/21 21:03:31 fuyu Exp $
+ * $Id: bulletin_board_ui.h,v 1.13 2003/11/22 19:17:16 fuyu Exp $
  */
 
 #ifndef _BULLETIN_BOARD_UI_H_
@@ -75,6 +75,9 @@ typedef struct _BulletinBoardGUIInfo
 
   GtkWidget *properties_dialog;
 
+  /* ᥤ򥹥ɽϤʸȤɽ뤫ɤĤΥǥե */
+  ThreadViewMailtoMode show_mailto_mode;
+
   /* Ǹν񤭹߻̾ */
   gchar *last_name;
 
