@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.25 2003/11/19 20:07:07 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.26 2003/11/22 19:17:16 fuyu Exp $
  */
 
 #include "config.h"
@@ -847,6 +847,8 @@ write_optional_prefs(FILE *file, gpointer user_data)
 				  show_threadlist_entry_when_thread_selected);
   OUTPUT_CONFIG_ATTRIBUTE_STRING(file, application, threadlist_view_contents);
 
+  OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, show_mailto_mode);
+
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, enable_popup_title);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, popup_title_delay);
 
@@ -929,6 +931,10 @@ configure_application(OchushaApplication *application)
       application->show_threadlist_entry_when_thread_selected = FALSE;
 
 
+      /* ɽϢ */
+      application->show_mailto_mode = OCHUSHA_DEFAULT_SHOW_MAILTO_MODE;
+
+
       /* 쥿Υݥåץå״Ϣ */
       application->enable_popup_title = OCHUSHA_DEFAULT_ENABLE_POPUP_TITLE;
       application->popup_title_delay = OCHUSHA_DEFAULT_POPUP_DELAY;
@@ -1101,6 +1107,13 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
     application->threadlist_view_contents
       = G_STRDUP(OCHUSHA_DEFAULT_THREADLIST_VIEW_CONTENTS);
 
+  application->show_mailto_mode
+    = (ThreadViewMailtoMode)ochusha_utils_get_attribute_int(pref_attrs,
+							   "show_mailto_mode");
+  if (application->show_mailto_mode < THREAD_VIEW_MAILTO_MODE_DEFAULT
+      || application->show_mailto_mode > THREAD_VIEW_MAILTO_MODE_HIDE)
+    application->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_DEFAULT;
+
   if (g_hash_table_lookup(pref_attrs, "enable_popup_title") != NULL)
     application->enable_popup_title
       = ochusha_utils_get_attribute_boolean(pref_attrs, "enable_popup_title");
