@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.20 2003/11/21 21:03:31 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.21 2003/11/22 19:17:16 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -90,6 +90,7 @@
 #define OCHUSHA_DEFAULT_HIDE_HIDDEN_BOARDLIST_ITEMS			FALSE
 #define OCHUSHA_DEFAULT_HIDE_HIDDEN_THREADS				FALSE
 
+#define OCHUSHA_DEFAULT_SHOW_MAILTO_MODE	THREAD_VIEW_MAILTO_MODE_DEFAULT
 
 
 #define OCHUSHA_GTKRC				"ochusha-gtkrc"
@@ -116,6 +117,13 @@ typedef struct _OchushaApplication OchushaApplication;
 
 #include "threadlist_filter.h"
 
+typedef enum
+{
+  THREAD_VIEW_MAILTO_MODE_DEFAULT = 0,
+  THREAD_VIEW_MAILTO_MODE_SHOW,
+  THREAD_VIEW_MAILTO_MODE_HIDE
+} ThreadViewMailtoMode;
+
 
 struct _OchushaApplication
 {
@@ -185,6 +193,12 @@ struct _OchushaApplication
   char *threadlist_view_contents;
 
 
+  /* ɽϢ */
+  /* ΤΥǥե */
+  /* ¾ĤΥǥեȤȥ̤Ѱդͽꡣ */
+  ThreadViewMailtoMode show_mailto_mode;
+
+
   /* 쥿Υݥåץå״Ϣ */
   gboolean enable_popup_title;
   guint popup_title_delay;
