@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.c,v 1.17 2003/11/05 07:54:50 fuyu Exp $
+ * $Id: threadlist_view.c,v 1.19 2003/11/07 15:52:40 fuyu Exp $
  */
 
 #include "config.h"
@@ -41,12 +41,18 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
+#ifndef HAVE_LOCALTIME_R
+#include <pthread.h>
+#endif
+
 #if DEBUG_WIDGET || DEBUG_WIDGET_MOST
 #include <stdio.h>
 #endif
 #include <stdlib.h>
 #include <string.h>
 
+#include <time.h>
+
 
 static void threadlist_view_class_init(ThreadlistViewClass *klass);
 static void popup_menu_callback(gpointer data, guint action,
@@ -63,7 +69,7 @@ static gboolean threadlist_view_title_not_equals(GtkTreeModel *model,
 						 GtkTreeIter *iter,
 						 gpointer search_data);
 
-static gboolean threadlist_view_move_cursor(ThreadlistView *view,
+static gboolean threadlist_view_move_cursor(GtkTreeView *tree_view,
 					    GtkMovementStep step,
 					    gint count);
 
@@ -86,9 +92,9 @@ static gboolean tree_view_button_release_event_cb(GtkWidget *widget,
 						  ThreadlistView *view);
 
 static void decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
-				       GtkCellRenderer *renderer,
-				       GtkTreeModel *tree_model,
-				       GtkTreeIter *iter, gpointer unused);
+				      GtkCellRenderer *renderer,
+				      GtkTreeModel *tree_model,
+				      GtkTreeIter *iter, gpointer unused);
 static void active_set_cell_data_flags(GtkTreeViewColumn *tree_column,
 				       GtkCellRenderer *renderer,
 				       GtkTreeModel *tree_model,
@@ -99,6 +105,11 @@ static void text_set_cell_data_rank_diff(GtkTreeViewColumn *tree_column,
 					 GtkTreeModel *tree_model,
 					 GtkTreeIter *iter,
 					 gpointer unused);
+static void text_set_cell_data_last_modified(GtkTreeViewColumn *tree_column,
+					     GtkCellRenderer *renderer,
+					     GtkTreeModel *tree_model,
+					     GtkTreeIter *iter,
+					     gpointer unused);
 
 
 GType
@@ -121,7 +132,7 @@ threadlist_view_get_type(void)
 	  (GInstanceInitFunc)threadlist_view_init,
 	};
 
-      tlv_type = g_type_register_static(GTK_TYPE_SCROLLED_WINDOW,
+      tlv_type = g_type_register_static(GTK_TYPE_TREE_VIEW,
 					"ThreadlistView", &tlv_info, 0);
     }
 
@@ -142,7 +153,6 @@ enum {
   ADVANCE_VIEW_SIGNAL,
   BACK_VIEW_SIGNAL,
 
-  MOVE_CURSOR_SIGNAL,
   INTERACTIVE_SEARCH_SIGNAL,
 
   DO_ON_SELECTION_SIGNAL,
@@ -161,15 +171,6 @@ enum {
 };
 
 
-enum {
-  TREE_VIEW_MOVE_CURSOR_SIGNAL,
-  TREE_VIEW_UNSELECT_ALL_SIGNAL,
-  TREE_VIEW_SELECT_CURSOR_ROW_SIGNAL,
-  TREE_VIEW_START_INTERACTIVE_SEARCH_SIGNAL,
-  TREE_VIEW_LAST_SIGNAL
-};
-
-
 enum {
   POPUP_MENU_OPEN_SELECTION = 1,
   POPUP_MENU_OPEN_SELECTION_IN_TAB,
@@ -187,10 +188,9 @@ struct _ThreadlistInfo
 };
 
 
-static GtkScrolledWindowClass *parent_class = NULL;
+static GtkTreeViewClass *parent_class = NULL;
 static gint threadlist_view_signals[LAST_SIGNAL] =
   { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-static gint tree_view_signals[TREE_VIEW_LAST_SIGNAL] = { 0, 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GQuark list_entry_info_id;
 
@@ -237,11 +237,32 @@ static const char *list_entry_background_color[16] =
 };
 
 
+#ifndef HAVE_LOCALTIME_R
+static pthread_mutex_t localtime_lock;
+
+#define LOCALTIME_LOCK					\
+  if (pthread_mutex_lock(&localtime_lock) != 0)		\
+    {							\
+      fprintf(stderr, "Couldn't lock a mutex.\n");	\
+      abort();						\
+    }
+
+#define LOCALTIME_UNLOCK				\
+  if (pthread_mutex_unlock(&localtime_lock) != 0)	\
+    {							\
+      fprintf(stderr, "Couldn't unlock a mutex.\n");	\
+      abort();						\
+    }
+
+#endif
+
+
 static void
 threadlist_view_class_init(ThreadlistViewClass *klass)
 {
   GObjectClass *o_class = (GObjectClass *)klass;
   GtkObjectClass *object_class = (GtkObjectClass *)klass;
+  GtkTreeViewClass *tree_view_class = (GtkTreeViewClass *)klass;
   GtkBindingSet *binding_set;
 
   /* ݥåץåץ˥塼ν */
@@ -322,6 +343,9 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
   /* GtkObject signals */
   object_class->destroy = threadlist_view_destroy;
 
+  /* GtkTreeView signals */
+  tree_view_class->move_cursor = threadlist_view_move_cursor;
+
   threadlist_view_signals[OPEN_VIEW_SIGNAL] =
     g_signal_new("open_view",
 		 G_TYPE_FROM_CLASS(klass),
@@ -409,16 +433,6 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 		 G_TYPE_BOOLEAN, 1,
 		 OCHUSHA_TYPE_BBS_THREAD);
 
-  threadlist_view_signals[MOVE_CURSOR_SIGNAL] =
-    g_signal_new("wrapped_move_cursor",
-		 G_TYPE_FROM_CLASS(klass),
-		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-		 G_STRUCT_OFFSET(ThreadlistViewClass, move_cursor),
-		 NULL, NULL,
-		 ochusha_marshal_BOOLEAN__ENUM_INT,
-		 G_TYPE_BOOLEAN, 2,
-		 GTK_TYPE_MOVEMENT_STEP,
-		 G_TYPE_INT);
   threadlist_view_signals[INTERACTIVE_SEARCH_SIGNAL] =
     g_signal_new("interactive_search",
 		 G_TYPE_FROM_CLASS(klass),
@@ -448,14 +462,6 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 		 G_TYPE_NONE, 1,
 		 G_TYPE_INT);
 
-  tree_view_signals[TREE_VIEW_MOVE_CURSOR_SIGNAL] =
-    g_signal_lookup("move_cursor", GTK_TYPE_TREE_VIEW);
-  tree_view_signals[TREE_VIEW_UNSELECT_ALL_SIGNAL] =
-    g_signal_lookup("unselect_all", GTK_TYPE_TREE_VIEW);
-  tree_view_signals[TREE_VIEW_SELECT_CURSOR_ROW_SIGNAL] =
-    g_signal_lookup("select_cursor_row", GTK_TYPE_TREE_VIEW);
-  tree_view_signals[TREE_VIEW_START_INTERACTIVE_SEARCH_SIGNAL] =
-    g_signal_lookup("start_interactive_search", GTK_TYPE_TREE_VIEW);
 
   /* Key bindings */
   gtk_binding_entry_add_signal(binding_set, GDK_o, 0, "do_on_selection", 1,
@@ -479,33 +485,33 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 			       G_TYPE_INT, BACK_VIEW);
 
   gtk_binding_entry_add_signal(binding_set, GDK_Up, 0,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
 			       G_TYPE_INT, -1);
   gtk_binding_entry_add_signal(binding_set, GDK_Down, 0,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
 			       G_TYPE_INT, 1);
 
   gtk_binding_entry_add_signal(binding_set, GDK_p, GDK_CONTROL_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
 			       G_TYPE_INT, -1);
   gtk_binding_entry_add_signal(binding_set, GDK_n, GDK_CONTROL_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_DISPLAY_LINES,
 			       G_TYPE_INT, 1);
 
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_CONTROL_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, 1);
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_MOD1_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, -1);
   gtk_binding_entry_add_signal(binding_set, GDK_v, GDK_MOD4_MASK,
-			       "wrapped_move_cursor", 2,
+			       "move_cursor", 2,
 			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
 			       G_TYPE_INT, -1);
 
@@ -550,15 +556,20 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
   klass->advance_view = NULL;
   klass->back_view = NULL;
 
-  klass->move_cursor = threadlist_view_move_cursor;
-  klass->interactive_search = NULL;
-
   klass->do_on_selection = threadlist_view_do_on_selection;
   klass->do_on_thread_at_cursor = threadlist_view_do_on_thread_at_cursor;
 
   list_entry_info_id
     = g_quark_from_static_string("ThreadlistView::ListEntryInfo");
 
+#ifndef HAVE_LOCALTIME_R
+  if (pthread_mutex_init(&localtime_lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a mutex.\n");
+      abort();
+    }
+#endif
+
   /*
    * XXX: ʲñʤǥեȤȤ
    *      桼ˤ񤭴ǽˤ٤
@@ -586,6 +597,7 @@ typedef enum
   THREADLIST_N_RESPONSES_READ,
   THREADLIST_N_RESPONSES_UNREAD,
   THREADLIST_RANK_DIFFERENCE,
+  THREADLIST_LAST_MODIFIED,
   THREADLIST_DATA,
   THREADLIST_N_COLUMNS
 } OchushaThreadlistItem;
@@ -594,7 +606,6 @@ typedef enum
 static void
 threadlist_view_init(ThreadlistView *view)
 {
-  GtkWidget *tree_view;
 #if 0
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
@@ -602,44 +613,26 @@ threadlist_view_init(ThreadlistView *view)
 
   GTK_WIDGET_SET_FLAGS(view, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
 
-  gtk_scrolled_window_set_policy(&view->container,
-				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-
-  tree_view = gtk_tree_view_new();
-  view->view = GTK_TREE_VIEW(tree_view);
-
-  gtk_scrolled_window_set_hadjustment(&view->container, GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0)));
-  gtk_scrolled_window_set_vadjustment(&view->container, GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0)));
-
-  /* MEMO: Τ褦GtkAdjustmentͿƤʤassertion˰óݤ:
-   * Gtk-CRITICAL **: file gtkrange.c: line 438 (gtk_range_get_adjustment):
-   * assertion 'GTK_IS_RANGE(range)' failed
-   * 2Ф롣
-   */
-  gtk_container_add(GTK_CONTAINER(&view->container), tree_view);
-
-  gtk_widget_show(tree_view);
-
-  view->selection = gtk_tree_view_get_selection(view->view);
+  view->selection = gtk_tree_view_get_selection(&view->view);
   gtk_tree_selection_set_mode(view->selection, GTK_SELECTION_MULTIPLE);
 
-  g_signal_connect(G_OBJECT(view->view), "motion_notify_event",
+  g_signal_connect(G_OBJECT(view), "motion_notify_event",
 		   G_CALLBACK(tree_view_motion_notify_event_cb), view);
-  g_signal_connect(G_OBJECT(view->view), "leave_notify_event",
+  g_signal_connect(G_OBJECT(view), "leave_notify_event",
 		   G_CALLBACK(tree_view_leave_notify_event_cb), view);
-  g_signal_connect(G_OBJECT(view->view), "button_press_event",
+  g_signal_connect(G_OBJECT(view), "button_press_event",
 		   G_CALLBACK(tree_view_button_press_event_cb), view);
-  g_signal_connect(G_OBJECT(view->view), "button_release_event",
+  g_signal_connect(G_OBJECT(view), "button_release_event",
 		   G_CALLBACK(tree_view_button_release_event_cb), view);
 
   view->model = NULL;
   view->model_is_opened = FALSE;
 
-  gtk_tree_view_set_search_column(view->view, THREADLIST_TITLE);
-  gtk_tree_view_set_search_equal_func(view->view,
+  gtk_tree_view_set_search_column(&view->view, THREADLIST_TITLE);
+  gtk_tree_view_set_search_equal_func(&view->view,
 				      threadlist_view_title_not_equals,
 				      NULL, NULL);
-  gtk_tree_view_set_enable_search(view->view, TRUE);
+  gtk_tree_view_set_enable_search(&view->view, TRUE);
   view->use_native_search = TRUE;
   view->being_searched = FALSE;
 }
@@ -776,9 +769,10 @@ threadlist_view_title_not_equals(GtkTreeModel *model, gint column,
 
 
 static gboolean
-threadlist_view_move_cursor(ThreadlistView *view, GtkMovementStep step,
+threadlist_view_move_cursor(GtkTreeView *tree_view, GtkMovementStep step,
 			    gint count)
 {
+  ThreadlistView *view = THREADLIST_VIEW(tree_view);
   gboolean result = FALSE;
 
   /*
@@ -801,13 +795,7 @@ threadlist_view_move_cursor(ThreadlistView *view, GtkMovementStep step,
       return TRUE;
     }
 
-  g_signal_emit(G_OBJECT(view->view),
-		tree_view_signals[TREE_VIEW_MOVE_CURSOR_SIGNAL],
-		0,
-		step, count,
-		&result);
-
-  return result;
+  return (*parent_class->move_cursor)(&view->view, step, count);
 }
 
 
@@ -830,12 +818,12 @@ threadlist_view_get_current_path(ThreadlistView *view)
   GtkTreePath *initial_path = NULL;
   GtkTreeIter iter;
 
-  gtk_tree_view_get_cursor(view->view, &initial_path, NULL);
+  gtk_tree_view_get_cursor(&view->view, &initial_path, NULL);
   if (initial_path != NULL)
     return initial_path;
 
   gtk_widget_grab_focus(GTK_WIDGET(view));
-  gtk_tree_view_get_cursor(view->view, &initial_path, NULL);
+  gtk_tree_view_get_cursor(&view->view, &initial_path, NULL);
   if (initial_path != NULL)
     return initial_path;
 
@@ -983,10 +971,10 @@ threadlist_view_find_next(ThreadlistView *view,
 	  if (path != NULL)
 	    gtk_tree_path_free(path);
 	  path = gtk_tree_model_get_path(view->model, &iter);
-	  gtk_tree_view_scroll_to_cell(view->view, path, NULL,
+	  gtk_tree_view_scroll_to_cell(&view->view, path, NULL,
 				       TRUE, 0.5, 0.0);
 	  gtk_tree_selection_select_iter(selection, &iter);
-	  gtk_tree_view_set_cursor(view->view, path, NULL, FALSE);
+	  gtk_tree_view_set_cursor(&view->view, path, NULL, FALSE);
 	  gtk_tree_path_free(path);
 
 	  return TRUE;
@@ -999,7 +987,7 @@ threadlist_view_find_next(ThreadlistView *view,
   if (path != NULL)
     gtk_tree_path_free(path);
 
-  gtk_tree_view_set_cursor(view->view, initial_path, NULL, FALSE);
+  gtk_tree_view_set_cursor(&view->view, initial_path, NULL, FALSE);
   gtk_tree_path_free(initial_path);
 
   return FALSE;
@@ -1012,7 +1000,7 @@ threadlist_view_set_enable_native_search(ThreadlistView *view,
 {
   g_return_if_fail(IS_THREADLIST_VIEW(view));
 
-  gtk_tree_view_set_enable_search(view->view, enable);
+  gtk_tree_view_set_enable_search(&view->view, enable);
   view->use_native_search = enable;
 }
 
@@ -1022,7 +1010,7 @@ threadlist_view_get_enable_native_search(ThreadlistView *view)
 {
   g_return_val_if_fail(IS_THREADLIST_VIEW(view), FALSE);
 
-  return gtk_tree_view_get_enable_search(view->view);
+  return gtk_tree_view_get_enable_search(&view->view);
 }
 
 
@@ -1034,12 +1022,9 @@ threadlist_view_interactive_search(ThreadlistView *view,
 
   if (view->use_native_search)
     {
-      gtk_widget_grab_focus(GTK_WIDGET(view->view));
+      gtk_widget_grab_focus(GTK_WIDGET(view));
 
-      g_signal_emit(G_OBJECT(view->view),
-		tree_view_signals[TREE_VIEW_START_INTERACTIVE_SEARCH_SIGNAL],
-		0,
-		&result);
+      return (*parent_class->start_interactive_search)(&view->view);
     }
   else
     {
@@ -1058,7 +1043,7 @@ threadlist_view_interactive_search(ThreadlistView *view,
 gboolean
 threadlist_view_start_interactive_search(ThreadlistView *view)
 {
-  g_return_val_if_fail(IS_THREADLIST_VIEW(view) && view->view != NULL, FALSE);
+  g_return_val_if_fail(IS_THREADLIST_VIEW(view), FALSE);
 
   return threadlist_view_interactive_search(view,
 					THREADLIST_SEARCH_DIRECTION_FORWARD);
@@ -1167,7 +1152,7 @@ threadlist_view_do_on_thread_at_cursor(ThreadlistView *view, gint what_to_do)
   GtkTreePath *path = NULL;
   gpointer data = NULL;
 
-  gtk_tree_view_get_cursor(view->view, &path, NULL);
+  gtk_tree_view_get_cursor(&view->view, &path, NULL);
   if (path == NULL)
     return;
 
@@ -1179,15 +1164,8 @@ threadlist_view_do_on_thread_at_cursor(ThreadlistView *view, gint what_to_do)
   if (data != NULL)
     {
       gboolean result = FALSE;
-      g_signal_emit(G_OBJECT(view->view),
-		    tree_view_signals[TREE_VIEW_UNSELECT_ALL_SIGNAL],
-		    0,
-		    &result);
-      g_signal_emit(G_OBJECT(view->view),
-		    tree_view_signals[TREE_VIEW_SELECT_CURSOR_ROW_SIGNAL],
-		    0,
-		    TRUE,
-		    &result);
+      gtk_tree_selection_unselect_all(view->selection);
+      (*parent_class->select_cursor_row)(&view->view, FALSE);
 
       switch (what_to_do)
 	{
@@ -1275,7 +1253,7 @@ tree_view_motion_notify_event_cb(GtkWidget *widget, GdkEventMotion *event,
   GtkTreeViewColumn *column;
   gpointer data = NULL;
 
-  if (!gtk_tree_view_get_path_at_pos(view->view, event->x, event->y, &path,
+  if (!gtk_tree_view_get_path_at_pos(&view->view, event->x, event->y, &path,
 				     &column, NULL, NULL))
     goto done;
 
@@ -1339,7 +1317,7 @@ tree_view_button_press_event_cb(GtkWidget *widget, GdkEventButton *event,
   gpointer data = NULL;
   gboolean result = FALSE;
 
-  if (!gtk_tree_view_get_path_at_pos(view->view, event->x, event->y, &path,
+  if (!gtk_tree_view_get_path_at_pos(&view->view, event->x, event->y, &path,
 				     &column, NULL, NULL))
     goto done;
 
@@ -1437,6 +1415,7 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
 		THREADLIST_N_RESPONSES_READ, thread->number_of_responses_read,
 		THREADLIST_N_RESPONSES_UNREAD, num_unread,
 		THREADLIST_RANK_DIFFERENCE, rank_diff,
+		THREADLIST_LAST_MODIFIED, ochusha_bbs_thread_get_last_modified_utc(thread),
 		THREADLIST_DATA, thread,
 		-1);
     }
@@ -1447,6 +1426,7 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
 		THREADLIST_N_RESPONSES, thread->number_of_responses_on_server,
 		THREADLIST_N_RESPONSES_READ, thread->number_of_responses_read,
 		THREADLIST_N_RESPONSES_UNREAD, num_unread,
+		THREADLIST_LAST_MODIFIED, ochusha_bbs_thread_get_last_modified_utc(thread),
 		THREADLIST_DATA, thread,
 		-1);
     }
@@ -1660,6 +1640,56 @@ text_set_cell_data_rank_diff(GtkTreeViewColumn *tree_column,
 }
 
 
+static void
+text_set_cell_data_last_modified(GtkTreeViewColumn *tree_column,
+				 GtkCellRenderer *renderer,
+				 GtkTreeModel *tree_model, GtkTreeIter *iter,
+				 gpointer view)
+{
+  GValue src = { 0, };
+  GValue value = { 0, };
+  time_t utc;
+
+  g_return_if_fail(gtk_list_store_iter_is_valid(GTK_LIST_STORE(tree_model),
+						iter));
+
+  gtk_tree_model_get_value(tree_model, iter, THREADLIST_LAST_MODIFIED, &src);
+  utc = g_value_get_long(&src);
+
+  g_value_init(&value, G_TYPE_STRING);
+  if (utc != 0)
+    {
+      gchar buffer[512];
+
+#ifdef HAVE_LOCALTIME_R
+      struct tm tm;
+      size_t result;
+      localtime_r(&utc, &tm);
+      result = strftime(buffer, 512, "%Y-%m-%d %T", &tm);
+      if (result == 0)
+	buffer[0] = '\0';
+#else
+      LOCALTIME_LOCK
+      {
+	struct tm *tm;
+	size_t result;
+	tm = localtime(&utc);
+	result = strftime(buffer, 512, "%Y-%m-%d %T", tm);
+	if (result == 0)
+	  buffer[0] = '\0';
+      }
+      LOCALTIME_UNLOCK;
+#endif
+      g_value_set_string(&value, buffer);
+    }
+  else
+    g_value_set_static_string(&value, "");
+
+  g_object_set_property((GObject *)renderer, "text", &value);
+  decorate_threadlist_entry(tree_column, renderer, tree_model, iter, view);
+}
+
+
 void
 threadlist_view_append_rank(ThreadlistView *view)
 {
@@ -1675,7 +1705,7 @@ threadlist_view_append_rank(ThreadlistView *view)
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
 					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
+  gtk_tree_view_append_column(&view->view, column);
   view->rank_column = column;
 
   view->view_is_initialized = TRUE;
@@ -1699,7 +1729,7 @@ threadlist_view_append_mark(ThreadlistView *view)
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  active_set_cell_data_flags,
 					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
+  gtk_tree_view_append_column(&view->view, column);
   view->mark_column = column;
 
   view->view_is_initialized = TRUE;
@@ -1721,7 +1751,7 @@ threadlist_view_append_title(ThreadlistView *view)
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
 					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
+  gtk_tree_view_append_column(&view->view, column);
   view->title_column = column;
 
   view->view_is_initialized = TRUE;
@@ -1744,7 +1774,7 @@ threadlist_view_append_number_of_responses(ThreadlistView *view)
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
 					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
+  gtk_tree_view_append_column(&view->view, column);
   view->n_res_column = column;
 
   view->view_is_initialized = TRUE;
@@ -1767,7 +1797,7 @@ threadlist_view_append_number_of_responses_read(ThreadlistView *view)
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
 					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
+  gtk_tree_view_append_column(&view->view, column);
   view->n_read_column = column;
 
   view->view_is_initialized = TRUE;
@@ -1790,7 +1820,7 @@ threadlist_view_append_number_of_responses_unread(ThreadlistView *view)
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
 					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
+  gtk_tree_view_append_column(&view->view, column);
   view->n_unread_column = column;
 
   view->view_is_initialized = TRUE;
@@ -1813,13 +1843,37 @@ threadlist_view_append_rank_difference(ThreadlistView *view)
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  text_set_cell_data_rank_diff,
 					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
+  gtk_tree_view_append_column(&view->view, column);
   view->rank_diff_column = column;
 
   view->view_is_initialized = TRUE;
 }
 
 
+void
+threadlist_view_append_last_modified(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view)
+		   && view->last_modified_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("Last Modified"),
+						    renderer, "text",
+						    THREADLIST_LAST_MODIFIED,
+						    NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  text_set_cell_data_last_modified,
+					  view, NULL);
+  gtk_tree_view_append_column(&view->view, column);
+  view->last_modified_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
 void
 threadlist_view_initialize_default_view(ThreadlistView *view)
 {
@@ -1857,9 +1911,10 @@ threadlist_view_open(ThreadlistView *view)
 			     G_TYPE_INT,	/* number of responses read */
 			     G_TYPE_INT,	/* number of responses unread*/
 			     G_TYPE_INT,	/* rank difference */
+			     G_TYPE_LONG,	/* last modified */
 			     G_TYPE_POINTER);	/* data (OchushaBBSThread *) */
 
-  gtk_tree_view_set_model(view->view, GTK_TREE_MODEL(store));
+  gtk_tree_view_set_model(&view->view, GTK_TREE_MODEL(store));
   view->model = GTK_TREE_MODEL(store);
 
   view->last_rank_diff = 0;
@@ -1969,8 +2024,8 @@ threadlist_view_scroll_to_thread(ThreadlistView *view,
   g_return_if_fail(thread == data);
 
   path = gtk_tree_model_get_path(view->model, &info->iter);
-  gtk_tree_view_scroll_to_cell(view->view, path, NULL, FALSE, 0.0f, 0.0f);
-  gtk_tree_view_set_cursor_on_cell(view->view, path, NULL, NULL, FALSE);
+  gtk_tree_view_scroll_to_cell(&view->view, path, NULL, FALSE, 0.0f, 0.0f);
+  gtk_tree_view_set_cursor_on_cell(&view->view, path, NULL, NULL, FALSE);
   gtk_tree_path_free(path);
 }
 
