@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.13 2003/11/20 03:14:37 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.15 2003/11/22 08:46:55 fuyu Exp $
  */
 
 #include "config.h"
@@ -32,6 +32,7 @@
 #include "ochusha.h"
 #include "ochusha_board_2ch.h"
 #include "ochusha_thread_2ch.h"
+#include "ochusha_utils_2ch.h"
 
 #include "htmlutils.h"
 #include "utils.h"
@@ -299,31 +300,38 @@ static char *
 ochusha_board_2ch_generate_base_path(OchushaBulletinBoard *board,
 				     const char *url)
 {
-  char *base_path = ochusha_utils_url_extract_http_absolute_path(url);
-  g_return_val_if_fail(base_path != NULL, NULL);
-
-  if (base_path[0] != '/')
-    {
-      G_FREE(base_path);
-      base_path = NULL;
-    }
-  g_return_val_if_fail(base_path != NULL, NULL);
-
   switch (board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
       {
-	char *tmp_pos = strchr(base_path + 1, '/');
+	char *path;
+	char *abs_path = ochusha_utils_url_extract_http_absolute_path(url);
+	char *tmp_pos;
+	g_return_val_if_fail(abs_path != NULL, NULL);
+
+	if (abs_path[0] != '/')
+	  {
+	    G_FREE(abs_path);
+	    return NULL;
+	  }
+	tmp_pos = strchr(abs_path + 1, '/');
+
 	if (tmp_pos != NULL)
-	  tmp_pos[1] = '\0';
-	break;
+	  {
+	    tmp_pos[1] = '\0';
+	    path = G_STRDUP(abs_path);
+	  }
+	else
+	  {
+	    path = G_STRDUP("/");
+	  }
+	G_FREE(abs_path);
+	return path;
       }
 
     default:
-      break;
+      return G_STRDUP("/");
     }
-
-  return base_path;
 }
 
 
@@ -332,8 +340,13 @@ ochusha_board_2ch_generate_board_id(OchushaBulletinBoard *board,
 				    const char *url)
 {
   char *id;
-  g_return_val_if_fail(board->base_path != NULL && board->base_path[0] == '/',
-		       NULL);
+  char *abs_path = ochusha_utils_url_extract_http_absolute_path(url);
+  if (abs_path != NULL && abs_path[0] != '/')
+    {
+      G_FREE(abs_path);
+      abs_path = NULL;
+    }
+  g_return_val_if_fail(abs_path != NULL, NULL);
 
   switch (board->bbs_type)
     {
@@ -342,43 +355,38 @@ ochusha_board_2ch_generate_board_id(OchushaBulletinBoard *board,
     case OCHUSHA_BBS_TYPE_MACHIBBS:
     case OCHUSHA_BBS_TYPE_MITINOKU:
       {
-	char *tmp_pos;
-	id = G_STRDUP(board->base_path + 1);
-	tmp_pos = strchr(id, '/');
+	char *tmp_pos = strchr(abs_path + 1, '/');
 	if (tmp_pos != NULL)
-	  *tmp_pos = '\0';
-	return id;
+	  {
+	    *tmp_pos = '\0';
+	  }
+	id = G_STRDUP(abs_path + 1);
+	break;
       }
 
     case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
       {
-	char *tmp_pos;
-	char *abs_path = ochusha_utils_url_extract_http_absolute_path(url);
-	g_return_val_if_fail(abs_path != NULL, NULL);
-
-	if (abs_path[0] != '/' || board->base_path == NULL)
-	  {
-	    G_FREE(abs_path);
-	    abs_path = NULL;
-	  }
-	g_return_val_if_fail(abs_path != NULL, NULL);
-
-	tmp_pos = strchr(abs_path + 1, '/');
-	if (tmp_pos != NULL)
-	  id = G_STRDUP(tmp_pos + 1);
+	char *tmp_pos = strchr(abs_path + 1, '/');
+	if (tmp_pos == NULL)
+	  id = G_STRDUP(abs_path + 1);
 	else
-	  id = G_STRDUP(board->base_path + 1);
-	G_FREE(abs_path);
-
-	tmp_pos = strchr(id, '/');
-	if (tmp_pos != NULL)
-	  *tmp_pos = '\0';
-	return id;
+	  {
+	    char *id_tail = strchr(tmp_pos + 1, '/');
+	    if (id_tail != NULL)
+	      *id_tail = '\0';
+	    id = G_STRDUP(tmp_pos + 1);
+	  }
+	break;
       }
 
     default:
-      return NULL;	/* ̤ݡ */
+      id = NULL;	/* ̤ݡ */
+      break;
     }
+
+  G_FREE(abs_path);
+
+  return id;
 }
 
 
@@ -400,35 +408,24 @@ static OchushaBBSThread *
 ochusha_board_2ch_lookup_thread_by_url(OchushaBulletinBoard *board,
 				       const char *url)
 {
-  OchushaBoard2ch *board_2ch;
-  OchushaBBSThread *thread;
-  char *thread_id;
-  char *server = ochusha_utils_url_extract_http_server(url);
-  char *path = ochusha_utils_url_extract_http_absolute_path(url);
-  char *tmp_pos;
-  char *res_part;
+  OchushaBBSThread *thread = NULL;
+  char *board_url = NULL;
+  char *thread_id = NULL;
 
-  g_return_val_if_fail(OCHUSHA_IS_BOARD_2CH(board)
-		       && url != NULL && path != NULL,
-		       NULL);
-  board_2ch = OCHUSHA_BOARD_2CH(board);
+  if (ochusha_utils_2ch_check_url(url, &board_url, NULL, NULL, &thread_id,
+				  NULL, NULL))
+    {
+      if (thread_id != NULL
+	  && g_ascii_strcasecmp(board->base_url, board_url) == 0)
+	thread = ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
+								thread_id);
+    }
 
-  g_return_val_if_fail((strncmp(path, "/test/read.cgi", 14) == 0), NULL);
+  if (board_url != NULL)
+    G_FREE(board_url);
 
-  tmp_pos = path + 14;			/* "/test/read.cgi"ɤФ */
-
-  g_return_val_if_fail(strcmp(board->server, server) == 0
-		       && strstr(tmp_pos, board->base_path) != tmp_pos,
-		       NULL);
-  tmp_pos += strlen(board->base_path);	/* base_pathʬɤФ */
-  g_return_val_if_fail(tmp_pos != NULL, NULL);
-
-  res_part = strchr(tmp_pos, '/');	/* ֹõ */
-  g_return_val_if_fail(res_part != NULL, NULL);
-
-  thread_id = G_STRNDUP(tmp_pos, res_part - tmp_pos);
-  thread = ochusha_bulletin_board_lookup_bbs_thread_by_id(board, thread_id);
-  G_FREE(thread_id);
+  if (thread_id != NULL)
+    G_FREE(thread_id);
 
   return thread;
 }
@@ -962,6 +959,8 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
 
   converter = iconv_open("UTF-8//IGNORE",
 			 ochusha_board_2ch_get_subback_html_encoding(board));
+  if (converter == (iconv_t)-1)
+    regfree(&threadlist_entry_pattern);
   g_return_val_if_fail(converter != (iconv_t)-1, FALSE);
 
   if (!ochusha_async_buffer_active_ref(buffer, "ochusha_board_2ch.c: ochusha_board_2ch_refresh_threadlist"))
@@ -970,6 +969,7 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
       fprintf(stderr, "buffer has been terminated.\n");
 #endif
       iconv_close(converter);
+      regfree(&threadlist_entry_pattern);
       return FALSE;
     }
 
@@ -1108,6 +1108,7 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
       g_slist_free(thread_list);
     }
 
+  regfree(&threadlist_entry_pattern);
   iconv_close(converter);
 
   return result;
