@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.12 2003/11/21 21:30:03 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.13 2003/11/22 16:57:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -523,21 +523,46 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
     {
       if (path_pos[12] == '/')
 	{
+	  /* JBBSФ../../bbs/read.cgi/ϤURL⤳ˤ뤫*/
 	  char *slash_pos = strchr(path_pos + 13, '/');
+	  fprintf(stderr, "slash_pos=%p\n", slash_pos);
 	  if (slash_pos != NULL)
 	    {
-	      int len = slash_pos - (path_pos + 12) -1;
-	      memcpy(tmp_buf, path_pos + 12, len);
-	      tmp_buf[len] = '\0';
-	      if (snprintf(board_url, PATH_MAX, "http://%s%s/", server,
-			   tmp_buf) < PATH_MAX)
+	      char *tmp_pos = strchr(slash_pos + 1, '/');
+	      int len;
+	      fprintf(stderr, "tmp_pos=%p\n", tmp_pos);
+	      if (tmp_pos != NULL && tmp_pos < (slash_pos + 10))
 		{
-		  board_id = G_STRDUP(tmp_buf);
-		  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
-		  bbs_type = OCHUSHA_BBS_TYPE_JBBS;	/* ̤ƥ */
+		  /* ɥۥåʵ§ġġ*/
+		  bbs_type = OCHUSHA_BBS_TYPE_JBBS_SHITARABA;
+		  *tmp_pos = '\0';
+		  board_id = G_STRDUP(slash_pos + 1);
+		  *tmp_pos = '/';
+		  slash_pos = tmp_pos;
 		}
 	      else
-		board_url[0] = '\0';
+		{
+		  /* ФǤʤJBBSäƸߤΤ*/
+		  bbs_type = OCHUSHA_BBS_TYPE_JBBS;
+		}
+	      len = slash_pos - (path_pos + 13);
+	      memcpy(tmp_buf, path_pos + 13, len);
+	      tmp_buf[len] = '\0';
+	      if (snprintf(board_url, PATH_MAX, "http://%s/%s/", server,
+			   tmp_buf) < PATH_MAX)
+		{
+		  if (board_id == NULL)
+		    board_id = G_STRDUP(tmp_buf);
+		  fprintf(stderr, "board_id=\"%s\"\n", board_id);
+		  fprintf(stderr, "tmp_buf=\"%s\"\n", tmp_buf);
+		  fprintf(stderr, "board_url=\"%s\"\n", board_url);
+		  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
+		}
+	      else
+		{
+		  bbs_type = OCHUSHA_BBS_TYPE_UNKNOWN;
+		  board_url[0] = '\0';
+		}
 #if 0
 	      fprintf(stderr, "  board_url: %s\n", board_url);
 #endif
@@ -715,7 +740,7 @@ extract_boards(OchushaBBSTable *table, OchushaBoardCategory *category,
       if (board == NULL)
 	{
 #if GUESS_2CH_BOARD_URL
-	  const char *base_path;
+	  const char *id;
 #endif
 	  if (name == NULL)
 	    {
@@ -728,10 +753,9 @@ extract_boards(OchushaBBSTable *table, OchushaBoardCategory *category,
 	    board->bbs_type = OCHUSHA_BBS_TYPE_MACHIBBS;
 
 #if GUESS_2CH_BOARD_URL
-	  base_path = ochusha_bulletin_board_get_base_path(board);
+	  id = ochusha_bulletin_board_get_id(board);
 	  /* ФΥ롼ľηǼĤϸʤġġ*/
-	  if (base_path[0] == '\0'
-	      || (base_path[0] == '/' && base_path[1] == '\0'))
+	  if (id == NULL || id[0] == '\0')
 	    {
 	      g_object_unref(G_OBJECT(board));
 	      board = NULL;
