@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_jbbs.c,v 1.8 2003/11/21 21:30:03 fuyu Exp $
+ * $Id: ochusha_utils_jbbs.c,v 1.9 2003/11/22 16:58:44 fuyu Exp $
  */
 
 #include "config.h"
@@ -743,7 +743,7 @@ ochusha_utils_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 		   number_of_cached_responses + 1,
 		   number_of_cached_responses == 0 ? "" : "&NOFIRST=TRUE")
 	  >= PATH_MAX)
-	return dat_buffer;
+	cgi_url[0] = '\0';
       break;
 
     case OCHUSHA_BBS_TYPE_MITINOKU:
@@ -757,18 +757,22 @@ ochusha_utils_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 		   thread->number_of_responses_on_server,
 		   number_of_cached_responses == 0 ? "" : "&nofirst=true")
 	  >= PATH_MAX)
-	return dat_buffer;
+	cgi_url[0] = '\0';
       break;
 
     default:	/* ̤ݡ */
-      return dat_buffer;
+      cgi_url[0] = '\0';
+      break;
     }
 
 #if DEBUG_NETWORK
   fprintf(stderr, "cgi_url=\"%s\"\n", cgi_url);
 #endif
 
-  html_buffer = ochusha_network_broker_read_from_cgi(broker, cgi_url);
+  if (cgi_url[0] != '\0')
+    html_buffer = ochusha_network_broker_read_from_cgi(broker, cgi_url);
+  else
+    html_buffer = NULL;
 
   if (html_buffer == NULL)
     {
@@ -777,6 +781,8 @@ ochusha_utils_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 	  gboolean signal_result;
 	  g_signal_emit_by_name(G_OBJECT(broker),
 				"access_failed", dat_buffer, &signal_result);
+	  ochusha_async_buffer_fix(dat_buffer,
+				   "ochusha_utils_jbbs.c: ochusha_utils_jbbs_fake_responses_source()");
 	}
       return dat_buffer;
     }
